// Initialization
var express = require('express');
var bodyParser = require('body-parser')
var path = require('path');
var phantomProxy = require('phantom-proxy');
var fs = require('fs');
var util = require('util');
var app = express();
var https = require('https');
var http = require('http');
var pem = require('pem');


// BI42 and leaflet URL homes
var leafletHomeURL = "https://<your BI Server hostname>/mis_leaflet/";
var leafletMapURL = "https://{s}.tile.osm.org/{z}/{x}/{y}.png";
var leafletImagesURL = "https://<your BI Server hostname>/images/map_icons/";
var mapAttribute = "Developed by #team name here#";

// Set max bytes for rendering
app.use(bodyParser.json({limit: '50mb', extended: true}));
app.use(bodyParser.urlencoded({limit: '50mb', extended: true}));

var urlencodedParser = bodyParser.urlencoded({ extended: true });
var CR = String.fromCharCode(13);

// HTTP area
// Set the port number, remark from here if using HTTPS
/*
app.listen(13000);
app.get("/", function(req, res) {
res.send("Server Up and Running");
});
*/
// Unremark above if using HTTP




// HTTPS area
// Remark from here if using HTTP
// Certificates created using CreateCertGUI.exe
var options = {
  key: fs.readFileSync('E:\\certs\\key-20190821-085009.pem'), 
  cert: fs.readFileSync('E:\\certs\\cert-20190821-085009.crt'),
  hostname : 'vmnodejs',
  path : "/",
  method : "GET",

};
// Create an HTTP service.
http.createServer(app).listen(8080);
// Create an HTTPS service identical to the HTTP service.
https.createServer(options, app).listen(13000);

//UnRemark above if using HTTPS

app.get("/", function(req, res) {
res.send("Server Up and Running");
});

app.get("/api/about", function(req, res) {
logInfo("About");
res.send("MIS Visualizations");
});

app.get("/api/formats", function(req, res) {
logInfo("Start Format");
var format = {"formats": ["text/html", "image/png"]};
res.setHeader('Content-Type', 'application/JSON');
res.send(JSON.stringify(format));
logInfo("Stop Format");
});

app.get("/api/visualizations", function(req, res) {
logInfo("Start Visualizations");
res.setHeader('Content-Type', 'application/JSON');
res.send(JSON.stringify(factory));
logInfo("Stop Visualizations");
});


app.get("/api/visualizations/:vizid/feeds", function(req, res) {
logInfo("Start feeds");
var viz = factory.getViz(req.params.vizid);
logInfo(" Viz ID => " + viz.name);


//Array of dimension objects
var feedArray = [];


feedArray.push(
{
"id": "category-latlong",
"name": viz.categoryLatLong,
"description": viz.categoryLatLong,
"axis": "0",
"type": "dimension",
"min": ""+viz.primaryValueMin+"",
"max": ""+viz.primaryValueMax+""
}
);

feedArray.push(
{
"id": "category-mapwidthheight",
"name": viz.categoryMapWidthHeight,
"description": viz.categoryMapWidthHeight,
"axis": "0",
"type": "dimension",
"min": ""+viz.primaryValueMin+"",
"max": ""+viz.primaryValueMax+""
}
);


feedArray.push(
{
"id": "category-mapzoom",
"name": viz.categoryMapZoom,
"description": viz.categoryMapZoom,
"axis": "0",
"type": "dimension",
"min": ""+viz.primaryValueMin+"",
"max": ""+viz.primaryValueMax+""
}
);

feedArray.push(
{
"id": "category-iconimgage",
"name": viz.categoryIconImage,
"description": viz.categoryIconImage,
"axis": "0",
"type": "dimension",
"min": ""+viz.primaryValueMin+"",
"max": ""+viz.primaryValueMax+""
}
);

feedArray.push(
{
"id": "category-iconsize",
"name": viz.categoryIconSize,
"description": viz.categoryIconSize,
"axis": "0",
"type": "dimension",
"min": ""+viz.primaryValueMin+"",
"max": ""+viz.primaryValueMax+""
}
);

feedArray.push(
{
"id": "category-markerdetail",
"name": viz.categoryMarkerDetail,
"description": viz.categoryMarkerDetail,
"axis": "0",
"type": "dimension",
"min": ""+viz.primaryValueMin+"",
"max": ""+viz.primaryValueMax+""
}
);

feedArray.push(
{
"id": "category-markerwidth",
"name": viz.categoryMarkerWidth,
"description": viz.categoryMarkerWidth,
"axis": "0",
"type": "dimension",
"min": ""+viz.primaryValueMin+"",
"max": ""+viz.primaryValueMax+""
}
);

feedArray.push(
{
"id": "category-doclink",
"name": viz.categoryDocLink,
"description": viz.categoryDocLink,
"axis": "0",
"type": "dimension",
"min": ""+viz.primaryValueMin+"",
"max": ""+viz.primaryValueMax+""
}
);


feedArray.push(
{
"id": "primary-values",
"name": viz.categoryIntensity,
"description": viz.categoryIntensity,
"axis": "0",
"type": "measure",
"min": ""+viz.primaryValueMin+"",
"max": ""+viz.primaryValueMax+""
}
);

var feeds = {
"feeds": feedArray
};
res.setHeader('Content-Type', 'application/JSON');
res.send(JSON.stringify(feeds));
logInfo("Stop feeds");
});


app.post("/api/visualizations/:vizid/render", function(req, res) {
logInfo("Start Render");
var viz = factory.getViz(req.params.vizid);
var html = viz.generate(req);
var height = req.body.height;
var width = req.body.width;
var format = req.headers.accept;
logInfo(" Viz ID => "+viz);

// Bitmap generation
if(format == "image/png") {
var self = this;
var randomnumber=Math.floor(Math.random()*101)
var path = 'html_page_'+randomnumber+'.html';
fs.writeFileSync(path, html);
phantomProxy.create(function(proxy) {
proxy.page.open(path, function (result) {
proxy.page.set('viewportSize', { width: width, height: height });
proxy.page.set('clipRect', { top: 0, left: 0, width: width, height: height });
proxy.page.waitForSelector("#chart", function (result){
proxy.page.renderBase64('PNG', function(result){
var bitmap = new Buffer(result, 'base64');
res.writeHead(200, {'Content-Type': 'image/png' });
res.end(bitmap, 'binary');
phantomProxy.end();
fs.unlinkSync(path); // Delete temp file
});
});
});
});
}

// HTML generation
else {
res.writeHead(200, {'Content-Type': 'text/html' });
res.end(html);
}
logInfo("Stop Render");
});

var VizFactory = function() {
this.visualizations = [];
this.getViz = function(id) {
for (var index = 0 ; index < this.visualizations.length ; index++) {
if (this.visualizations[index].id === id) return this.visualizations[index];
}
};
};

var Visualization = function(id, name, catLatLong,  catMapWidthHeight, catMapZoom,   catIconImgage, catIconSize, catMarkerDetail, catMarkerWidth, catDocLink, catHeatIntensity, valMin, valMax, myDesc)

{
	this.id = id;
	this.name = name;
	this.categoryLatLong = catLatLong;
	this.categoryMapWidthHeight = catMapWidthHeight;
	this.categoryMapZoom = catMapZoom;
	this.categoryIconImage = catIconImgage;
	this.categoryIconSize = catIconSize;
	this.categoryMarkerDetail = catMarkerDetail;
	this.categoryMarkerWidth = catMarkerWidth;
	this.categoryDocLink = catDocLink;
	this.categoryIntensity = catHeatIntensity;
	this.primaryValueMin = valMin;
	this.primaryValueMax = valMax;
	this.description = myDesc;
};

// ################ Start of Marker Map Details ########################
var vizCEMapMarker = new Visualization("cemarkermap", "Marker", "Latitude,Longitude" , "Map Width,Height", "Map Zoom", "Icon Image", "Icon Size", "Marker Detail" , "Marker Detail Width", "Document URL", "Intensity Measure", 1, 1,"MARKER MAP - Every selection on the right must have a dimension object, measure or custom element mapped to it. NOTE: Overlapping markers only the last marker will be displayed, thus consider using CLUSTER map.")

vizCEMapMarker.generate = function(req) {
var w = req.body.width ? req.body.width : 800;
var h = req.body.height ? req.body.height : 800;
var dimLatLong = getData(req, getFeedings(req, "category-latlong")[0].dataId);
var dimMapWH = getData(req, getFeedings(req, "category-mapwidthheight")[0].dataId);
var dimMapZoom = getData(req, getFeedings(req, "category-mapzoom")[0].dataId);
var dimIconImg = getData(req, getFeedings(req, "category-iconimgage")[0].dataId);
var dimIconSize = getData(req, getFeedings(req, "category-iconsize")[0].dataId);
var dimDataMark = getData(req, getFeedings(req, "category-markerdetail")[0].dataId);
var dimDataMarkWidth = getData(req, getFeedings(req, "category-markerwidth")[0].dataId);
var dimDataDocLink = getData(req, getFeedings(req, "category-doclink")[0].dataId);
var mesData = getData(req, getFeedings(req, "primary-values")[0].dataId);


var values = [
['Latitude,Longitude', 'Map Width,Height' , 'Map Height', 'Map Zoom' ,'Icon Image', 'Icon Size', 'Marker Detail', 'Marker Detail Width','Document Link', 'Heat Intensity']
];

for (var i = 0; i < dimLatLong.values.rawvalues.length; i++) {
values.push(

			[
				dimLatLong.values.rawvalues[i], // Array = 0
				dimMapWH.values.rawvalues[i], // Array = 1
				dimMapZoom.values.rawvalues[i], // Array = 2
				dimIconImg.values.rawvalues[i], // Array = 3
				dimIconSize.values.rawvalues[i], // Array = 4
				dimDataMark.values.rawvalues[i],  // Array = 5
				dimDataMarkWidth.values.rawvalues[i],  // Array = 6
				dimDataDocLink.values.rawvalues[i],  // Array = 7
				
			]
		);
}

var centreMap = values[1][0] ;

var mapWH = values[1][1];
var arrmapWH = mapWH.split(",");
var mapW = parseInt(arrmapWH[0],10);
var mapH = parseInt(arrmapWH[1],10);
var mapZ = parseInt(values[1][2],10);
var nmapW = parseInt(values[1][6],10);
var cmap = (mapW - nmapW) - 5 ;
var datapointcount = values.length - 1;
var iconSize = parseInt(values[1][4], 10);
var newMapW = mapW + nmapW;

var html =
'<!DOCTYPE html>' + CR +
'<html style="height:100%; margin:auto; >' + CR +
' <head>' + CR +
' </head>' + CR +
' <body style="border:0; margin:0; padding:0; height:100%; vertical-align:middle; display:table-cell">' + CR +
' <script src="' + leafletHomeURL + 'leaflet.js"></script>' + CR +
' <link rel="stylesheet" href="' + leafletHomeURL + 'leaflet.css" />' + CR +

' <table align = "left" border="1"> <tr>' + CR +
' <td id="tab1" style="width:' + mapW + 'px; height:'+ mapH +'px;">' + CR +
' <div id="map" style="width: ' + mapW + 'px; height: ' + mapH +'px;"></div>' + CR +
' <script type="text/javascript">' + CR +
' {' + CR +
'  var map = L.map("map").setView([' + centreMap + '], ' + mapZ + ');' + CR +
'  L.tileLayer("' + leafletMapURL + '", {maxZoom: 20,attribution: "' + mapAttribute + '"}).addTo(map);' + CR +
' var markerGroup = L.featureGroup().addTo(map);' + CR +
' var marker, myText , myDocLink;' + CR ;

// Start adding points to map here from the array
for (var j = 1; j < values.length; j++) {

		varLatLong = values[j][0];
		varIconIMG = values[j][3];
		varMark = values[j][5];
		varDocLink = values[j][7];
		

html +=
' var myIcon = L.icon({iconUrl: " ' + leafletImagesURL + varIconIMG + '",iconSize: [ ' + iconSize + ', ' + (iconSize ) + '],iconAnchor: [0, 0],popupAnchor: [0, -5]});' + CR +

' marker = L.marker([' + varLatLong + '],{icon:  myIcon}).addTo(markerGroup);' + CR +
' marker.myText = "' + varMark + '"; marker.myDocLink = "' + varDocLink + '"; ' + CR ;

}

html +=
'	function markerClick(event) { ' + CR +
'	var markerTable = event.layer.myText; ' + CR +
'	var doclink = event.layer.myDocLink ; ' + CR +
'	var displink = "Data points = ' + datapointcount +'" ;' + CR +
'	if (doclink) {displink = "<a href=\'" + doclink + "\' target=\'_blank\'>Open document link</a>"; }' + CR + 

'	document.getElementById("urllink").innerHTML = displink;' + CR + 

'	if (markerTable) { ' + CR +
'   	document.getElementById("tab1").style.width = "' + cmap + 'px";' + CR +
'   	document.getElementById("map").style.width = "' + cmap + 'px"; ' + CR +
'		document.getElementById("details").style.width = "' + nmapW + 'px"; ' + CR +
'		document.getElementById("details").style.display = "block"; ' + CR +
'		document.getElementById("mapmarker").innerHTML =  markerTable;' + CR +
'		}' + CR +
'	};' + CR +
'	function markerClose()' + CR +
'	{' + CR +
'		document.getElementById("details").style.display = "none"; '+ CR + 
'		document.getElementById("tab1").style.width = "' + mapW + 'px";' + CR + 
'		document.getElementById("map").style.width = "' + mapW + 'px";' + CR + 
'	}' + CR +
'	markerGroup.on("click", markerClick); ' + CR +
'}	'+ CR +

' </script></td>' + CR +
' <td valign="top" id="details" style="display:none;"> ' + CR +
' <table style="height:' + mapH + 'px; width:' + nmapW + ';" border="1"> ' + CR +
' <tr><td bgcolor="#729ce0"align="left">Marker Details</td><td align="center" bgcolor="Red"><a href="" onclick="markerClose();">X</a></td></tr>' + CR +

' <tr><td valign="top" style="width:' + nmapW + 'px; height:100%;" colspan="2" id="mapmarker"></td></tr><tr><td bgcolor="#729ce0" align="center" id="urllink" colspan="2"></td></tr></tr></table> ' + CR +
' </body>' + CR +
' </html>' + CR;
return html;
}
// ################ End of Marker Map Details ########################



// ################ Start of Cluster Map Details ########################
var vizCEMapMarkerCluster = new Visualization("cemarkermapcluser", "Cluster", "Latitude,Longitude" , "Map Width,Height", "Map Zoom", "Icon Image", "Icon Size", "Marker Detail" , "Marker Detail Width", "Document URL", "Intensity Measure", 1, 1,"CLUSTER MAP - Every selection on the right must have a dimension object, measure or custom element mapped to it.")

vizCEMapMarkerCluster.generate = function(req) {
var w = req.body.width ? req.body.width : 800;
var h = req.body.height ? req.body.height : 800;
var dimLatLong = getData(req, getFeedings(req, "category-latlong")[0].dataId);
var dimMapWH = getData(req, getFeedings(req, "category-mapwidthheight")[0].dataId);
var dimMapZoom = getData(req, getFeedings(req, "category-mapzoom")[0].dataId);
var dimIconImg = getData(req, getFeedings(req, "category-iconimgage")[0].dataId);
var dimIconSize = getData(req, getFeedings(req, "category-iconsize")[0].dataId);
var dimDataMark = getData(req, getFeedings(req, "category-markerdetail")[0].dataId);
var dimDataMarkWidth = getData(req, getFeedings(req, "category-markerwidth")[0].dataId);
var dimDataDocLink = getData(req, getFeedings(req, "category-doclink")[0].dataId);
var mesData = getData(req, getFeedings(req, "primary-values")[0].dataId);


var values = [
['Latitude,Longitude', 'Map Width,Height', 'Map Zoom' ,'Icon Image', 'Icon Size', 'Marker Detail', 'Marker Detail Width','Document Link', 'Heat Intensity']
];

for (var i = 0; i < dimLatLong.values.rawvalues.length; i++) {
values.push(

			[
				dimLatLong.values.rawvalues[i], // Array = 0
				dimMapWH.values.rawvalues[i], // Array = 1
				dimMapZoom.values.rawvalues[i], // Array = 2
				dimIconImg.values.rawvalues[i], // Array = 3
				dimIconSize.values.rawvalues[i], // Array = 4
				dimDataMark.values.rawvalues[i],  // Array = 5
				dimDataMarkWidth.values.rawvalues[i],  // Array = 6
				dimDataDocLink.values.rawvalues[i],  // Array = 7
				
			]
		);
}

var centreMap = values[1][0];

var mapWH = values[1][1];
var arrmapWH = mapWH.split(",");
var mapW = parseInt(arrmapWH[0],10);
var mapH = parseInt(arrmapWH[1],10);

var mapZ = parseInt(values[1][2],10);
var nmapW = parseInt(values[1][6],10);
var cmap = (mapW - nmapW) - 5 ;
var datapointcount = values.length - 1;
var iconSize = parseInt(values[1][4], 10);

var newMapW = mapW + nmapW;

var html =
'<!DOCTYPE html>' + CR +
'<html style="height:100%; margin:auto; >' + CR +
' <head>' + CR +
' </head>' + CR +
' <body style="border:0; margin:0; padding:0; height:100%; vertical-align:middle; display:table-cell">' + CR +
' <script src="' + leafletHomeURL + 'leaflet.js"></script>' + CR +
' <script src="' + leafletHomeURL + 'leaflet.markercluster-src.js"></script>' + CR +
' <link rel="stylesheet" href="' + leafletHomeURL + 'leaflet.css" />' + CR +
' <link rel="stylesheet" href="' + leafletHomeURL + 'screen.css" />' + CR +
' <link rel="stylesheet" href="' + leafletHomeURL + 'MarkerCluster.css" />' + CR +
' <link rel="stylesheet" href="' + leafletHomeURL + 'MarkerCluster.Default.css" />' + CR +

' <table align = "left" border="1"> <tr>' + CR +
' <td id="tab1" style="width:' + mapW + 'px; height:'+ mapH +'px;">' + CR +
' <div id="map" style="width: ' + mapW + 'px; height: ' + mapH +'px;"></div>' + CR +

' <script type="text/javascript">' + CR +
' {' + CR +
'  var map = L.map("map").setView([' + centreMap + '], ' + mapZ + ');' + CR +
'  L.tileLayer("' + leafletMapURL + '", {maxZoom: 20,attribution: "' + mapAttribute + '"}).addTo(map);' + CR +
' var marker, myText , myDocLink;' + CR +
' var markers = new L.MarkerClusterGroup(); ' +CR +
' var markersList = []; ' + CR 

// Start adding points to map here from the array
for (var j = 1; j < values.length; j++) {

		varLatLong = values[j][0];
		varIconIMG = values[j][3];
		varMark = values[j][5];
		varDocLink = values[j][7];

html +=
' var myIcon = L.icon({iconUrl: " ' + leafletImagesURL + varIconIMG + '",iconSize: [ ' + iconSize + ', ' + (iconSize ) + '],iconAnchor: [0, 0],popupAnchor: [0, -5]});' + CR +

'  var myLat = L.latLng(' + varLatLong + '); var marker = new L.Marker(myLat, {icon:myIcon}); ' + CR +
'  marker.myText = "' + varMark + '"; marker.myDocLink = "' + varDocLink + '"; markersList.push(marker);markers.addLayer(marker); ' + CR ;

}

html +=
'   map.addLayer(markers);' + CR +
'	markers.on("click", function (event) { ' + CR +
'	var markerTable = event.layer.myText; ' + CR +
'	var doclink = event.layer.myDocLink ; ' + CR +
'	var displink = "Data points = ' + datapointcount +'" ;' + CR +
'	if (doclink) {displink = "<a href=\'" + doclink + "\' target=\'_blank\'>Open document link</a>"; }' + CR + 
'	document.getElementById("urllink").innerHTML = displink;' + CR + 
'	if (markerTable) { ' + CR +
'   	document.getElementById("tab1").style.width = "' + cmap + 'px";' + CR +
'   	document.getElementById("map").style.width = "' + cmap + 'px"; ' + CR +
'		document.getElementById("details").style.width = "' + nmapW + 'px"; ' + CR +
'		document.getElementById("details").style.display = "block"; ' + CR +
'		document.getElementById("mapmarker").innerHTML =  markerTable ; ' + CR +
'		} ' + CR +
'	});' + CR +
'	function markerClose()' + CR +
'	{' + CR +
'		document.getElementById("details").style.display = "none"; '+ CR + 
'		document.getElementById("tab1").style.width = "' + mapW + 'px";' + CR + 
'		document.getElementById("map").style.width = "' + mapW + 'px";' + CR + 
'	}' + CR +

'}	'+ CR +

' </script></td>' + CR +
' <td valign="top" id="details" style="display:none;"> ' + CR +
' <table style="height:' + mapH + 'px; width:' + nmapW + ';" border="1"> ' + CR +
' <tr><td bgcolor="#729ce0"align="left">Marker Details</td><td align="center" bgcolor="Red"><a href="" onclick="markerClose();">X</a></td></tr>' + CR +

' <tr><td valign="top" style="width:' + nmapW + 'px; height:100%;" colspan="2" id="mapmarker"></td></tr><tr><td bgcolor="#729ce0" align="center" id="urllink" colspan="2"></td></tr></tr></table> ' + CR +
' </body>' + CR +
' </html>' + CR;
return html;
}
// ################ End of Cluster Map Details ########################


// ################ Start of Heat Map #########################

var vizCEMapHeat = new Visualization("cemarkermapHeat", "Heat", "Latitude,Longitude", "Map Width,Height", "Map Zoom", "Icon Image", "Icon Size", "Marker Detail" , "Marker Detail Width" , "Document URL", "Intensity Measure", 1, 1,"HEAT MAP - Every selection on the right must have a dimension object, measure or custom element mapped to it.")

vizCEMapHeat.generate = function(req) {
var w = req.body.width ? req.body.width : 800;
var h = req.body.height ? req.body.height : 800;
var dimLatLong = getData(req, getFeedings(req, "category-latlong")[0].dataId);
var dimMapWH = getData(req, getFeedings(req, "category-mapwidthheight")[0].dataId);
var dimMapZoom = getData(req, getFeedings(req, "category-mapzoom")[0].dataId);
var dimIconImg = getData(req, getFeedings(req, "category-iconimgage")[0].dataId);
var dimIconSize = getData(req, getFeedings(req, "category-iconsize")[0].dataId);
var dimDataMark = getData(req, getFeedings(req, "category-markerdetail")[0].dataId);
var dimDataMarkWidth = getData(req, getFeedings(req, "category-markerwidth")[0].dataId);
var dimDataDocLink = getData(req, getFeedings(req, "category-doclink")[0].dataId);
var mesData = getData(req, getFeedings(req, "primary-values")[0].dataId);


var values = [
['Latitude,Longitude', 'Map Width,Height', 'Map Zoom' ,'Icon Image', 'Icon Size', 'Marker Detail', '"Marker Detail Width', 'Document Link', 'Heat Intensity']
];

for (var i = 0; i < dimLatLong.values.rawvalues.length; i++) {
values.push(

			[
				dimLatLong.values.rawvalues[i], // Array = 0
				dimMapWH.values.rawvalues[i], // Array = 1
				dimMapZoom.values.rawvalues[i], // Array = 2
				mesData.values.rawvalues[i] // Array = 3
			]
		);
}

var centreMap = values[1][0] ;
var mapWH = values[1][1];
var arrmapWH = mapWH.split(",");
var mapW = parseInt(arrmapWH[0],10);
var mapH = parseInt(arrmapWH[1],10);
var mapZ = values[1][2];
var mapI = values[1][3];
var html =
'<!DOCTYPE html>' + CR +
'<html style="height:100%; margin:auto; display:table">' + CR +
' <head>' + CR +
' </head>' + CR +
' <body style="border:0; margin:0; padding:0; height:100%; vertical-align:middle; display:table-cell">' + CR +
' <script src="' + leafletHomeURL + 'leaflet.js"></script>' + CR +
' <script src="' + leafletHomeURL + 'leaflet-heat.js"></script>' + CR +
' <link rel="stylesheet" href="' + leafletHomeURL + 'leaflet.css" />' + CR +
' <div id="map" style="width: ' + mapW + 'px; height: ' + mapH +'px"></div>' + CR +

' <script type="text/javascript">' + CR +
' {' + CR +
'  var map = L.map("map").setView([' + centreMap + '], ' + mapZ + ');' + CR +
'  L.tileLayer("' + leafletMapURL + '", {maxZoom: 20,attribution: "' + mapAttribute + '"}).addTo(map);' + CR +

// Start adding points to map here from the array
'  var markers = [' + CR ;

for (var j = 1; j < values.length; j++) {

		varLatLong = values[j][0];

html +=

'[' + varLatLong + ',' + mapI + '], ';
}

html +=
' [0,0,0]   ];' + CR + 
' var heat = L.heatLayer(markers).addTo(map);' + CR + 
' setTimeout(function () {  map.fitBounds(heat.getBounds());}, 1000); ' + CR +
' }' + CR +
' </script>' + CR +
' </body>' + CR +
' </html>' + CR;
return html;
}
// ################ End of Heat Map #########################

// ################ Start of Square Map #########################

var vizCEMapSquare = new Visualization("cemarkermapSquare", "Square", "Latitude,Longitude" ,"Map Width,Height", "Map Zoom", "Icon Image", "Icon Size", "Marker Detail" , "Marker Detail Width" , "Document URL", "Intensity Measure", 1, 1,"SQUARES MAP - Every selection on the right must have a dimension object, measure or custom element mapped to it. Use Icon size for metre squared, and Icon Image for square colour")

vizCEMapSquare.generate = function(req) {
var w = req.body.width ? req.body.width : 800;
var h = req.body.height ? req.body.height : 800;
var dimLatLong = getData(req, getFeedings(req, "category-latlong")[0].dataId);
var dimMapWH = getData(req, getFeedings(req, "category-mapwidthheight")[0].dataId);
var dimMapZoom = getData(req, getFeedings(req, "category-mapzoom")[0].dataId);
var dimIconImg = getData(req, getFeedings(req, "category-iconimgage")[0].dataId);
var dimIconSize = getData(req, getFeedings(req, "category-iconsize")[0].dataId);
var dimDataMark = getData(req, getFeedings(req, "category-markerdetail")[0].dataId);
var dimDataMarkWidth = getData(req, getFeedings(req, "category-markerwidth")[0].dataId);
var dimDataDocLink = getData(req, getFeedings(req, "category-doclink")[0].dataId);
var mesData = getData(req, getFeedings(req, "primary-values")[0].dataId);


var values = [
['Latitude,Longitude', 'Map Width,Height', 'Map Zoom' ,'Icon Image', 'Icon Size', 'Marker Detail', '"Marker Detail Width', 'Document Link', 'Heat Intensity']
];

for (var i = 0; i < dimLatLong.values.rawvalues.length; i++) {
values.push(

			[
				dimLatLong.values.rawvalues[i], // Array = 0
				dimMapWH.values.rawvalues[i], // Array = 1
				dimMapZoom.values.rawvalues[i], // Array = 2
				dimIconImg.values.rawvalues[i], // Array = 3
				dimIconSize.values.rawvalues[i], // Array = 4
				dimDataMark.values.rawvalues[i]  // Array = 5
				
			]
		);
}

var centreMap = values[1][0] ;
var mapWH = values[1][1];
var arrmapWH = mapWH.split(",");
var mapW = parseInt(arrmapWH[0],10);
var mapH = parseInt(arrmapWH[1],10);
var mapZ = values[1][2];
var mapL = values[1][4];
var html =
'<!DOCTYPE html>' + CR +
'<html style="height:100%; margin:auto; display:table">' + CR +
' <head>' + CR +
' </head>' + CR +
' <body style="border:0; margin:0; padding:0; height:100%; vertical-align:middle; display:table-cell">' + CR +
' <script src="' + leafletHomeURL + 'leaflet.js"></script>' + CR +
' <script src="' + leafletHomeURL + 'leaflet-heat.js"></script>' + CR +
' <link rel="stylesheet" href="' + leafletHomeURL + 'leaflet.css" />' + CR +
' <div id="map" style="width: ' + mapW + 'px; height: ' + mapH +'px"></div>' + CR +

' <script type="text/javascript">' + CR +
' {' + CR +
'  var map = L.map("map").setView([' + centreMap + '], ' + mapZ + ');' + CR +
'  L.tileLayer("' + leafletMapURL + '", {maxZoom: 20,attribution: "' + mapAttribute + '"}).addTo(map);' + CR ;

// Start adding points to map here from the array

for (var j = 1; j < values.length; j++) {

		varLatLong = values[j][0];
		varColor = values[j][3];
		varPopup= values[j][5];
		if (varColor) {} else {varColor="#fa0202";}
		

html +=

'	var circle = new L.Circle([' + varLatLong + '], ' + mapL + ' , {color: "",fillColor: "",	fillOpacity: 0.0}).addTo(map); ' + CR +
'	var rectangle = new L.Rectangle(circle.getBounds(), {color: "' + varColor + '",fillColor: "' + varColor + '",	fillOpacity: 0.2}).addTo(map).bindPopup("'+ varPopup +'"); ' + CR ;

}

html +=

' }' + CR +
' </script>' + CR +
' </body>' + CR +
' </html>' + CR;
return html;
}
// ################ End of Square Map #########################


var factory = new VizFactory();
factory.visualizations.push(vizCEMapMarker);
factory.visualizations.push(vizCEMapMarkerCluster);
factory.visualizations.push(vizCEMapHeat);
factory.visualizations.push(vizCEMapSquare);


function getFeedings(req, id) {
for (var i = 0 ; i < req.body.feeding.length ; i++) {
if (req.body.feeding[i].id === id) {
return req.body.feeding[i].expressions;
}
}
};

function getData(req, id) {
for (var i = 0 ; i < req.body.data.length ; i++) {
if (req.body.data[i].id == id) {
return req.body.data[i];
}
}
};

function distinct(arr) {
var values = [];
for (var i = 0 ; i < arr.length ; i++) {
var value = arr[i];
if (values.indexOf(value) === -1) {
values.push(value);
}
}
return values;
};

function logInfo(info){
var currentdate = new Date();
console.log(currentdate.toLocaleString() + " => " + info);
};